%% Initialization and Parameter Setting
% clear all 
function [Error] = CuringModelError(Coeffs,TempValue,ConeCenter,height)
    % User Inputs--------------------------------------------------------------
    load('R20Data.mat')
    TempValue = TempData20;
    ConeCenter = center20;
    height = height20;
    CuredData = CuredData20;
    
    Coeffs
   
    tEnd = 29;               % [s] Duration of exposure simulation
    tStep = 5;              % [s] Step time of simulation
    
    [numFrames imWidth imHeight] = size(TempValue);
    
    shrink = 2;
    distPerPix = 0.0264*shrink;      % [mm] Distance per pixel
    rscaling = 1.54/shrink;          % image pixels/projector pixels. 

    
    A = Coeffs(1);
    B = Coeffs(2);
    C = Coeffs(3);
    Q = Coeffs(4);
    sigma_heat = Coeffs(5)/distPerPix;
    H = Coeffs(6);
%     A = 0.0087;            % [1/um]    ** Not sure if these units need scaling!!!
%     B = 0.20;             % [1/um]    ** Not sure if these units need scaling!!!
%     C = 0.08;               % [cm^2/mJ] ** Not sure if these units need scaling!!!
% 
%     Q = 0.09;               % heat conversion coefficient
%     sigma_heat = .08/distPerPix;       % heat spread (related to specific heat) 
%     H = 0.1 ;               % Heat Curing Efficiency
    
    
    RoomTemperature = 30.33; % Calculated from data

    sigma_scattering = 0.0; % Scattering coefficient (added to alpha)
    sigma_defocus = 0.05/distPerPix;    % Blur due to defocus

    plotEandC = 0;          % 1 for animation of curing pattern, 0 for no E and C plot (slightly faster)
    ShowFinalStructure = 0;
    CuringThreshold = 70;

    
    zeroPadding = 50;
    resolutionx = 151;      % # pixels in x
    resolutiony = imHeight;    % # pixels in y
    resolutionz = imWidth+zeroPadding;      % # pixels in z
    
    cutoff = 0.8;           % Cutoff in cure% in length calculation; Range = [0 1]
    % End User Input-----------------------------------------------------------

    %Initialization of other matrices------------------------------------------

    I = zeros(resolutionx,resolutiony,resolutionz);
    IvertP = zeros(resolutionx,resolutiony,resolutionz);
    M_old = zeros(resolutionx,resolutiony,resolutionz);
    RateOfHeatEvolution = zeros(resolutionx,resolutiony,resolutionz);
    HeatGenerated = RoomTemperature*ones(resolutionx,resolutiony,resolutionz);

    alpha = (A+B)*ones(resolutionx,resolutiony,resolutionz);
    E = zeros(resolutionx,resolutiony,resolutionz);
    %% Initialize Light sources and apply to grid
    SourceNum = 0; 

    %Light Sources ------------------------------------------------------------
    % Explaination of light source placement: 
    % The light sources are simplified as a constant intensity at the border of 
    % the meshgrid. Initially, light sources are assigned to their positions 
    % and the border pixels are never changed throughout the simulation. 
    %
    % It's important that the sources are placed at the border! 
    % (1 or 'resolution') Otherwise they will get rewritten. 
    % Remember that C and E maps are plotted in Imagesc, which flips x and y.
    % Cannot enter 0 as a location--enter 1!!

    % % %Create Light Sources 
    radius = 20; %25*rscaling
    center = [ 75 ConeCenter];
    [X Y] = meshgrid(-radius:1:radius);
    I(center(1)-radius:center(1)+radius,center(2)-radius:center(2)+radius,1) = 10*(sqrt(X.^2+Y.^2)<= radius);

    % clearvars X Y
    % radius = 25*rscaling;
    % center = [50 175]
    % [X Y] = meshgrid(-radius:1:radius);
    % I(center(1)-radius:center(1)+radius,center(2)-radius:center(2)+radius,1) = 10*(sqrt(X.^2+Y.^2)<= radius);
    % 
    % clearvars X Y
    % radius = 30*rscaling;
    % center = [50 325]
    % [X Y] = meshgrid(-radius:1:radius);
    % I(center(1)-radius:center(1)+radius,center(2)-radius:center(2)+radius,1) = 10*(sqrt(X.^2+Y.^2)<= radius);
    % 
    if(plotEandC == 1)
        figure(1)
        clf
        imshow(I(:,:,1))
        title('Illumination Pattern')
    end
    %% Light Propagation Simulation 
    if(plotEandC == 1)
        figure(3)
        colormap parula
        clf
    end
    t=0;

    while(t<tEnd)
        alpha = A*exp(-C*E) + B;
        for h = 2:resolutionz-1         % z translation
           I(:,:,h) = imgaussfilt(I(:,:,h-1),sigma_defocus,'Padding','replicate').*exp(-(alpha(:,:,h)+sigma_scattering).*distPerPix);
        end
        E = E + sqrt(I.^2)*tStep;
        subplot(3,1,1)
        M_new = 1 - exp(-C.*(1+H*(HeatGenerated-RoomTemperature)).*E);
        RateOfHeatEvolution = (1/Q).*(M_new - M_old)./tStep;
                 % z translation
        HeatGenerated(:,:,:) = tStep*RateOfHeatEvolution(:,:,:) + imgaussfilt3(HeatGenerated(:,:,:), sigma_heat,'Padding','replicate');

        M_old = M_new;
        Cured = 100*(1 - exp(-C.*(1+H*(HeatGenerated-RoomTemperature)).*E));

        Eslice(:,:) = E(center(1)-radius,:,:);    
        Curedslice(:,:) = Cured(center(1),:,:);
        HeatGeneratedslice(:,:) = HeatGenerated(center(1)-radius,:,:);
        if (plotEandC == 1)
            subplot(3,1,1)
            imagesc(Eslice)
            subplot(3,1,2)
            imagesc(HeatGeneratedslice)
            axis equal
            subplot(3,1,3)
            imagesc(im2bw(Curedslice/100,0.7))
            pause(0.005)
        end
        if (mod(t,5)==0&&t>0)
%             figure(1)
%             subplot(1,3,1)
%             Data = imrotate(squeeze(CuredData(t/5,:,:)),-90);
%             imshow(Data)
%             subplot(1,3,2)
%             imshow(im2bw(Curedslice/100,0.7))
%             subplot(1,3,3)
%             imshow((Data-im2bw(Curedslice/100,0.7)).^2);
%             pause(1)
           [minval idx] = min((Cured(center(1),center(2),:)-CuringThreshold).^2);
           Error1(t/5) = mean(mean(([imrotate(squeeze(CuredData(t/5,:,:)),-90) zeros(imHeight,zeroPadding)]-im2bw(Curedslice/100,0.7)).^2));
        end
%         figure(2)
%         imshow(imrotate(squeeze(TempValue(t+1,:,:)),-90),[]
        Error2(t/tStep+1) = mean(mean(([imrotate(squeeze(TempValue(t+1,:,:)),-90) RoomTemperature*ones(imHeight,zeroPadding)] - (HeatGeneratedslice)).^2));
%         pause(1)
%         figure(3)
        t = t + tStep;
%     t
    end
    Error1;
    Error2;
    Error = norm([10.*Error2 Error1])

    % Show Final C and E plots 
    if (plotEandC == 1)

        subplot(3,1,1)
    %     axis equal
        title('Total Dose')
        colorbar
        subplot(3,1,2)
    %     axis equal
        title('Heat')
        colorbar
        subplot(3,1,3)
    %     axis equal
        title('Percent Cured')
        colorbar

    end

    %%
    if(ShowFinalStructure ==1)
        TopPoint = zeros(resolutionx,resolutiony);
        HeatPoint = zeros(resolutionx,resolutiony);
        for h = 3:resolutionz
            [idx_x idx_y] = find(Cured(:,:,h)>=CuringThreshold);
            if (length(idx_x)==0)
                break
            end
            for k=1:length(idx_x)
                TopPoint(idx_x(k),idx_y(k)) = h*distPerPix;
                HeatPoint(idx_x(k),idx_y(k)) = HeatGenerated(idx_x(k),idx_y(k),h);
            end
    %         figure(2)
    %         subplot(1,3,1)
    %         imshow(Cured(:,:,h),[0 100])
    %         subplot(1,3,2)
    %         scatter(idx_y,idx_x,'.')
    %         axis([1 resolutionx 1 resolutiony])
    %         axis equal
    %         subplot(1,3,3)
    %         imshow(TopPoint,[0 100])
    %         pause(0.1)
        end

        [x y] = meshgrid(linspace(1,resolutionx*distPerPix,resolutionx),linspace(1,resolutiony*distPerPix,resolutiony));
        z = TopPoint';
        c = HeatPoint';
        figure(5)
        surf(x,y,z)
        axis vis3d
        axis equal
        lighting phong
        shading interp
        colorbar EastOutside
        colormap parula
        title('Final Structure')
    end
    
end