function [Cured, TipLocation] =ProcessImages(RadiusNumber,ShowData)
% RadiusNumber = 25;
% ShowData = 1

    numPics = 6;
    SE0 = strel('disk',4);
    offset = 0.3;
    SE4 = strel('disk',0);
    filestart = strcat('R',num2str(RadiusNumber),'_images\');
    for i=1:numPics
        numStr = num2str(5*(i-1));
        filename = strcat(filestart,numStr,'.jpg');
        image = imread(filename);
        if(i == 1)
            FirstPic = image(:,:,3);
            imageSum = im2double(FirstPic);
            imageSum= imageSum./max(max(imageSum));
            level = graythresh(imageSum);
            BWSum(i,:,:) = imclose(imopen(im2bw(imageSum,level+offset),SE0),SE4);
        else
            currentImage = im2double(rgb2gray(image-previousImage));
            imageSum = imageSum.*((i-1)/i) + currentImage;
            level = graythresh(imageSum);
            BWSum(i,:,:) = squeeze(BWSum(i-1,:,:))|imclose(imopen(im2bw(imageSum,level+offset),SE0),SE4);
            if(i == numPics)
            LastPic = image(:,:,3);
            end
        end
        previousImage = image;
    end
    [height width color] = size(FirstPic);


    %% Find Growth Region

    difference = LastPic-FirstPic;
    differenceBW = imclose(im2bw(difference,graythresh(difference)+0.2),ones(8));
    RegionOfChange = imopen(imclose(differenceBW,ones(50)),ones(5));
    clearvars difference differenceBW

    %Identify the region that changes the most between frames
    RegionsLabeled = bwlabel(RegionOfChange);
    RegionOfInterest = mode(RegionsLabeled(RegionsLabeled>0));
    [RegionChange_x RegionChange_y] = find(RegionsLabeled == RegionOfInterest);
    scaling = 1:floor(length(RegionChange_x)/100):length(RegionChange_x);

    %Find the region associated with the area that changes the most, set that
    %as the growth region
    RegionsLabeledAll = bwlabel(squeeze(BWSum(numPics,:,:)));
    A = RegionsLabeledAll(RegionChange_x(scaling),RegionChange_y(scaling));
    GrowthRegion = mode(mode(A(A>0)));
    [GrowthRegion_x GrowthRegion_y] = find(RegionsLabeledAll == GrowthRegion);
    Minx = min(GrowthRegion_x);
    Maxx = max(GrowthRegion_x);
    Miny = min(GrowthRegion_y);
    Maxy = max(GrowthRegion_y);

    height_ROI = Maxy-Miny+1;
    width_ROI = Maxx-Minx+1;

    %% Find Growth Direction

    NoValuesUntil = 1;
    for i = 1:numPics
        [idx_x idx_y] = find(squeeze(BWSum(i,Minx:Maxx,Miny:Maxy)));
        if(length(idx_x) ==0)
            NoValuesUntil = i+1;
        end
        center(i,:) = [mean(idx_y) mean(idx_x)];
    end

    yData = center(NoValuesUntil:numPics,1);
    xData = center(NoValuesUntil:numPics,2);
    [p] = polyfit(xData,yData,1);

    X = linspace(1,width_ROI-5,2);
    Y = polyval(p,X);
    Rotation = atand(p(1));

    SE1 = strel('disk',1);
    SE2 = strel('disk',2);

    Cured = zeros(numPics,width_ROI,height_ROI);
    for i = 1:numPics
        if(i>1)
        Cured(i,:,:) = Cured(i-1,:,:);
        end
        dummy0 = squeeze(BWSum(i,:,:));
        dummy1 = rotateAround(dummy0,Minx+(width_ROI),Miny+(height_ROI/2),-Rotation,'bicubic');
        dummy2 = imopen(dummy1(Minx:Maxx,Miny:Maxy),SE1);
        dummy3 = bwlabel(dummy2);
        CuredRegion = mode(dummy3(dummy3>0));
        [x y] = find(dummy3 == CuredRegion);
    %     clearvars dummy0 dummy1 dummy2 dummy3
        for h=1:length(x)
            Cured(i,x(h),y(h)) = 1;
        end
        Cured(i,:,:) = imclose(squeeze(Cured(i,:,:)),SE2);
    %     subplot(1,numPics,i)
    %     imshow(squeeze(Cured(i,:,:)));

        BrightnessVariation = smooth(sum(Cured(i,:,:)),5);
        [Max(i) MaxLoc(i)] = max(BrightnessVariation);

    end
    TipLocation = mean(MaxLoc);



    %% Plot Data 

    if(ShowData ==1)
        figure(1)
        clf

        % Display the Growth Region
        subplot(1,2,1)
        border = 100
        imshow(LastPic(Minx-border:Maxx+border,Miny-border:Maxy+border))
        line([Miny Miny]-Miny+border,[Minx Maxx]-Minx+border,'LineWidth',2,'Color','y')
        line([Maxy Maxy]-Miny+border,[Minx Maxx]-Minx+border,'LineWidth',2,'Color','y')
        line([Miny Maxy]-Miny+border,[Maxx Maxx]-Minx+border,'LineWidth',2,'Color','y')
        line([Miny Maxy]-Miny+border,[Minx Minx]-Minx+border,'LineWidth',2,'Color','y')
        title('Growth Region')

        % Display the Growth Direction
        subplot(1,2,2)
        imshow(LastPic(Minx-border:Maxx+border,Miny-border:Maxy+border))
        hold on
        plot(Y+border,X+border,'r','lineWidth',3)
        plot(Y(1)+border,X(1)+border,'r^','markerSize',10,'markerFaceColor','r')
        title('Growth Direction')

        figure(2)
        for i = 1:numPics
            subplot(1,numPics,i)
            imshow(squeeze(Cured(i,:,:)));
        end
        hold on 
        plot(TipLocation,1,'r.','MarkerSize',20)

    end

end