%% Initialize and get data
% function [TempValue, TipLocationX] = ProcessThermalImages(RadiusNumber,MinTemp,MaxTemp,ShowData)
    RadiusNumber = 30;
    MinTemp = 20;
    MaxTemp = 55;
    ShowData = 1;

    filename = strcat('R',num2str(RadiusNumber),'_30s_1sSpacing.wmv');
    vidobj = VideoReader(filename);

    numFrames = get(vidobj,'NumberOfFrames');
    PicLim_x = 1037;

    PicsToSample = 3:5:28;
    lastFrame = 33;
    numFrames = 30;
    frame = im2double(read(vidobj, 1));
    [height width color] = size(frame);
    TempScale = frame(:,PicLim_x:width,1);
    
    for i = 3:lastFrame
        frame = im2double(read(vidobj, i));
        frames(i-2,:,:,:) = frame(:,1:PicLim_x,:);
    end

    level = graythresh(frames(numFrames,:,:,:));
    for i = 1:numFrames
        Frame(:,:,:) = frames(i,:,:,:);
        BWframes(i,:,:) = im2bw(Frame,level+0.1);
    end

    %% Find resin growth region

    difference = squeeze(BWframes(26,:,:))-squeeze(BWframes(3,:,:));
    [idxx idxy] = find(difference<0);
    difference(idxx,idxy)=0;
    Region = imopen(difference,ones(3));
%     figure(1)
%     clf
%     imshow(Region)
    [RegionIdx_x RegionIdx_y] = find(Region);

    Minx = min(RegionIdx_x);
    Maxx = max(RegionIdx_x);
    Miny = min(RegionIdx_y);
    Maxy = max(RegionIdx_y);

    height_ROI = Maxy-Miny+1;
    width_ROI = Maxx-Minx+1;
    for i = 1:numFrames
        BW_ROI(i,:,:) = BWframes(i,Minx:Maxx,Miny:Maxy);
    end

    %% Find Resin Growth Direction and Angle of growth
    NoValuesUntil = 1;
    for i = 1:numFrames

        [idx_x idx_y] = find(squeeze(BW_ROI(i,:,:)));
        if(length(idx_x) ==0)
            NoValuesUntil = i+1;
        end
        center(i,:) = [mean(idx_y) mean(idx_x)];

    end

    yData = center(NoValuesUntil:numFrames,1);
    xData = center(NoValuesUntil:numFrames,2);
    [p] = polyfit(xData,yData,1);
    Rotation = atand(p(1));
    X = linspace(1,width_ROI-5,2);
    Y = polyval(p,X);
    
    
    %% Align and locate tip 

    k = 1;
    for i = 1:numFrames  
        Rot = rotateAround(squeeze(frames(i,:,:,1)),Minx+(width_ROI),Miny+(height_ROI/2),-Rotation,'bicubic');
        ZoomRot(i,:,:) = Rot(Minx:Maxx,Miny-10:Maxy+10);
        TempValue(i,:,:) = ZoomRot(i,:,:)*(MaxTemp-MinTemp+1)+MinTemp;

        BrightnessVariation = smooth(sum(TempValue(i,:,:)),5);

        if(var(BrightnessVariation) > 10)
            [Max(k) MaxLoc(k)] = max(BrightnessVariation);
            k = k+1;
        end
    
    
    end
    TipLocationX = mean(MaxLoc);
    
    %% Display Plots

    if(ShowData == 1)
        figure(3)
        clf
        NonThresh(:,:) = frames(numFrames,:,:);
        subplot(1,2,1)
        border = 100;
        imshow(NonThresh(Minx-border:Maxx+border,Miny-border:Maxy+border),[])
        line([Miny Miny]-Miny+border,[Minx Maxx]-Minx+border,'LineWidth',2,'Color','y')
        line([Maxy Maxy]-Miny+border,[Minx Maxx]-Minx+border,'LineWidth',2,'Color','y')
        line([Miny Maxy]-Miny+border,[Maxx Maxx]-Minx+border,'LineWidth',2,'Color','y')
        line([Miny Maxy]-Miny+border,[Minx Minx]-Minx+border,'LineWidth',2,'Color','y')
        title('Growth Region')

        subplot(1,2,2)
        imshow(NonThresh(Minx-border:Maxx+border,Miny-border:Maxy+border),[])
        hold on
        plot(Y+border,X+border,'r','lineWidth',3)
        plot(Y(1)+border,X(1)+border,'r^','markerSize',10,'markerFaceColor','r')
        title('Growth Direction')

        figure(4)
        clf
%         PicSpacing = 1:5:29
        for i = 1:6
            subplot(2,3,i)
            imshow(squeeze(TempValue(5*(i-1)+1,:,:)),[])
            colorbar
        end
        hold on 
        plot(TipLocationX, 1,'r.','MarkerSize',25)
    end

end